package com.simber.market.controllers;

import com.simber.market.common.security.authenticate.Authenticate;
import com.simber.market.common.security.authorization.Authorization;
import com.simber.market.common.security.user.UserSession;
import com.simber.market.common.security.user.models.UserRole;
import com.simber.market.common.security.user.models.UserSessionDetails;
import com.simber.market.dto.CreateOrUpdateProductParams;
import com.simber.market.dto.IncreaseProductParams;
import com.simber.market.dto.ProductDto;
import com.simber.market.mappers.ProductEntityMapper;
import com.simber.market.services.ProductAuthorizationService;
import com.simber.market.services.ProductService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.Collection;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/Products")
public class ProductController {

    private final ProductService productService;
    private final ProductEntityMapper productEntityMapper;

    private final ProductAuthorizationService productAuthorizationService;

    @PostMapping
    @Authenticate(roles = {UserRole.Admin, UserRole.Seller})
    public ProductDto create(@RequestBody CreateOrUpdateProductParams params, @UserSession UserSessionDetails userSessionDetails) {
        return productEntityMapper.toDto(productService.create(userSessionDetails.getId(), params));
    }

    @Authenticate
    @PutMapping("{id}")
    public void update(@PathVariable Long id, @RequestBody CreateOrUpdateProductParams params, @UserSession UserSessionDetails userSession) {
        if (!productAuthorizationService.accessDeleteProductById(id, userSession)) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN);
        }
        productService.update(id, params);
    }

    @GetMapping("{id}")
    @Authenticate
    public ProductDto getById(@PathVariable Long id) {
        return productEntityMapper.toDto(productService.getById(id));
    }

    @Authenticate
    @DeleteMapping("{id}")
    public void deleteById(@PathVariable Long id, @UserSession UserSessionDetails userSession) {
        if (!productAuthorizationService.accessDeleteProductById(id, userSession)) {
            throw new ResponseStatusException(HttpStatus.FORBIDDEN);
        }
        productService.delete(id);
    }

    @GetMapping
    @Authenticate
    public Collection<ProductDto> getAll(
            @RequestParam(required = false) String name,
            @RequestParam(required = false) Collection<String> tags
    ) {
        return productService.getAll(name, tags).stream().map(productEntityMapper::toDto).toList();
    }

    @Authenticate(roles = UserRole.Admin)
    @PostMapping("{id}/Increase")
    public void increase(
            @PathVariable Long id,
            @RequestBody IncreaseProductParams params
    ) {
        productService.increase(id, params);
    }

    @Authenticate(roles = UserRole.Admin)
    @PostMapping("{id}/Decrease")
    public void decrease(
            @PathVariable Long id,
            @RequestBody IncreaseProductParams params
    ) {
        productService.decrease(id, params);
    }

    @Authenticate
    @PostMapping("{id}/Increase/Service")
    public void serviceIncrease(
            @PathVariable Long id,
            @RequestBody IncreaseProductParams params
    ) {
        productService.increase(id, params);
    }

    @Authenticate
    @PostMapping("{id}/Decrease/Service")
    public void serviceDecrease(
            @PathVariable Long id,
            @RequestBody IncreaseProductParams params
    ) {
        productService.decrease(id, params);
    }
}
