package com.simber.market.services;

import com.simber.market.dto.CreateOrUpdateProductParams;
import com.simber.market.dto.IncreaseProductParams;
import com.simber.market.entities.ProductEntity;
import com.simber.market.entities.ProductTagEntity;
import com.simber.market.entities.ProductTagEntityId;
import com.simber.market.repositories.ProductRepository;
import com.simber.market.repositories.ProductTagRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class ProductServiceImpl implements ProductService {

    private final ProductRepository productRepository;
    private final ProductTagRepository productTagRepository;

    @Override
    @Transactional(readOnly = true)
    public Collection<ProductEntity> getAll(String name, Collection<String> tags) {
        return productRepository.findAllActive().stream().filter(item -> {
            boolean isValid = true;

            if (name != null && !name.isEmpty()) {
                isValid = item.getName().contains(name);
            }

            if (isValid && tags != null && !tags.isEmpty()) {
                isValid = item.getTags().stream().anyMatch(i -> tags.stream().anyMatch(j -> i.getId().getName().equals(j)));
            }

            return isValid;

        }).toList();
    }

    @Override
    @Transactional(readOnly = true)
    public ProductEntity getById(Long id) {
        return productRepository.findByIdActive(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND, "Product not found"));
    }

    @Override
    @Transactional
    public ProductEntity create(Long ownerId, CreateOrUpdateProductParams params) {
        final var hospital = new ProductEntity();
        hospital.setName(params.name());
        hospital.setDescription(params.description());
        hospital.setOwnerId(ownerId);
        hospital.setPrice(params.price());
        final var savedHospital = productRepository.save(hospital);
        updateTags(savedHospital, params.tags());
        return savedHospital;
    }

    @Override
    @Transactional
    public void update(Long id, CreateOrUpdateProductParams params) {
        final var hospital = getById(id);
        hospital.setName(params.name());
        hospital.setDescription(params.description());
        hospital.setPrice(params.price());
        productRepository.save(hospital);
        updateTags(hospital, params.tags());
    }

    @Override
    @Transactional
    public void delete(Long id) {
        var product = getById(id);
        product.setIsDeleted(true);
        productRepository.save(product);
    }

    @Override
    @Transactional
    public void increase(Long productId, IncreaseProductParams params) {
        if (params.amount() <= 0) throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Amount invalid");

        var product = getById(productId);
        product.setAmount(product.getAmount() + params.amount());
        productRepository.save(product);
    }

    @Override
    @Transactional
    public void decrease(Long productId, IncreaseProductParams params) {
        if (params.amount() <= 0) throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Amount invalid");

        var product = getById(productId);
        product.setAmount(product.getAmount() - params.amount());
        if (product.getAmount() < 0) throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Amount invalid");
        productRepository.save(product);
    }

    private void updateTags(ProductEntity product, Set<String> rooms) {
        if (rooms != null && !rooms.isEmpty()) {
            Set<String> existingRoomNames = product.getTags().stream()
                    .map(h -> h.getId().getName())
                    .collect(Collectors.toSet());

            List<ProductTagEntity> roomsToAdd = rooms.stream()
                    .filter(roomName -> !existingRoomNames.contains(roomName))
                    .map(roomName -> {
                        var hospitalRoomEntity = new ProductTagEntity();
                        var id = new ProductTagEntityId();
                        id.setProduct(product);
                        id.setName(roomName);
                        hospitalRoomEntity.setId(id);
                        return hospitalRoomEntity;
                    })
                    .collect(Collectors.toList());

            List<ProductTagEntity> roomsToRemove = product.getTags().stream()
                    .filter(room -> !rooms.contains(room.getId().getName()))
                    .collect(Collectors.toList());

            if (!roomsToAdd.isEmpty()) {
                productTagRepository.saveAll(roomsToAdd);
            }

            if (!roomsToRemove.isEmpty()) {
                productTagRepository.deleteAll(roomsToRemove);
            }
        }
    }
}
