package com.simber.purchase.services;

import com.simber.purchase.common.security.context.SecurityContextHolder;
import com.simber.purchase.dto.CancelPurchaseParams;
import com.simber.purchase.dto.CreatePurchaseParams;
import com.simber.purchase.entities.PurchaseEntity;
import com.simber.purchase.product.client.ProductClient;
import com.simber.purchase.product.client.params.IncreaseProductParams;
import com.simber.purchase.repositories.PurchaseRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.util.Collection;
import java.util.Objects;
import java.util.Random;

@Service
@RequiredArgsConstructor
public class PurchaseServiceImpl implements PurchaseService {

    private final PurchaseRepository purchaseRepository;

    @Lazy
    @Autowired
    private ProductClient productClient;

    private final Random random = new Random();

    @Override
    @Transactional
    public PurchaseEntity create(Long userId, CreatePurchaseParams params) {
        var purchase = new PurchaseEntity();
        purchase.setAmount(params.amount());
        purchase.setProductId(params.productId());
        purchase.setPaymentId(random.nextLong());
        purchase.setUserId(userId);
        purchase.setStatus("Expectations");
        purchase = purchaseRepository.save(purchase);
        productClient.decrease(params.productId(), new IncreaseProductParams(params.amount()), SecurityContextHolder.getContext().getAccessToken());
        return purchase;
    }

    @Override
    @Transactional
    public void cancel(CancelPurchaseParams params) {
        var purchase = getById(params.purchaseId());
        purchase.setStatus("Canceled");
        productClient.increase(purchase.getProductId(), new IncreaseProductParams(purchase.getAmount()), SecurityContextHolder.getContext().getAccessToken());
    }

    @Override
    @Transactional(readOnly = true)
    public PurchaseEntity getById(Long id) {
        return purchaseRepository.findById(id)
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND, "Purchase not found"));
    }

    @Override
    @Transactional(readOnly = true)
    public Collection<PurchaseEntity> getAll(Long userId, Integer offset, Integer limit) {
        if (userId != null)
            return purchaseRepository.findAllByUserIdSlice(userId, PageRequest.of(offset, limit)).getContent();

        return purchaseRepository.findAllSlice(PageRequest.of(offset, limit)).getContent();
    }

    @Override
    @Transactional
    public void updateStatusToPaid(Long paymentId) {
        var entity = getById(paymentId);

        if (Objects.equals(entity.getStatus(), "Canceled")) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST);
        }
        entity.setStatus("Paid");
        purchaseRepository.save(entity);
    }
}
